//===========================================================================
//
//  Filename  : taskview.cpp
//  Purpose   : Task editor Widget for QtGantt
//  Copyright : Copyright (C) 2000 Olivier Berrod <obe@alcove.fr>
//  Original  : by Gunnstein Lye
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//===========================================================================

#include <stdio.h>
#include <stream.h>

#include <qlistview.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qevent.h>
#include <qsplitter.h>
#include <qstring.h>

 
#include "task.h"
#include "project.h"
#include "taskview.h"

TaskView::TaskView ( Project *proj, QWidget *parent, const char *name )
    : QWidget( parent, name )
{

	project = proj;


	// Get the heignt of the widget.
	int fh = this->height();

	splitter = new QSplitter( this );

	// Task view creation

	QFontMetrics fm( fontMetrics() );
	listview = new QListView( splitter, "gantt table" );

	listview->addColumn("Task ID");
	listview->setColumnWidthMode(0, QListView::Maximum);
	listview->addColumn("Description");
	listview->setColumnWidthMode(1, QListView::Manual);
	listview->setColumnAlignment(1, AlignLeft);
	listview->addColumn("Duration");
	listview->setColumnWidthMode(2, QListView::Maximum);
	listview->setColumnAlignment(2, AlignRight);
	// The middle column is truncated to fit between the left and right
	// columns
	int two_outsides = listview->columnWidth(0) + listview->columnWidth(2);
	// KLUDGE! the width of the main widget is hard-coded to 790 in main.cpp, 
	// and the listview is half of that. I haven't been able to determine
	// the width of the listview at all - listview->width() gives 100, which
	// just ain't right.
	listview->setColumnWidth(1, (790/2) - two_outsides);
	listview->setMinimumSize( 400, this->height() );

	listview->setFocusPolicy( StrongFocus );
	listview->setAllColumnsShowFocus( TRUE );
	listview->setSorting( -1 );

	// Detail view creation
	detailview = new TaskField( splitter, "gantt table" );
	detailview->setBackgroundColor("white");
	detailview->setMinimumSize( 400, this->height() );
}

TaskView::~TaskView ()
{
	if (listview)
		delete listview;
	if (detailview)
		delete detailview;
}

void TaskView::populate ()
{
	int  cpt;
	// Qstring for set text in QListViewItem
	QString  task_nb; 
	QString  task_de; 	

	listview->clear();

	connect(listview, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(handleClick(QListViewItem *)));
	// populating the list
	if ( this->project )
	{
		for (cpt=(project->numtasks );cpt-- ; cpt >1  )
		{
			task_nb.sprintf("%d",project->tasks[cpt]->num);
			task_de.sprintf("%d",project->tasks[cpt]->durationlength);
		
			new QListViewItem( listview,task_nb, *project->tasks[cpt]->description,task_de );
		}
	}
}



void TaskView::redraw()
{

	listview->setMinimumSize( 400,this->height());
	detailview->setMinimumSize( (this->width()-400),this->height() );
	//	listview->repaint();
	this->populate();
}

// Slot
void TaskView::handleClick(QListViewItem * item)
{

    debug("click handled\n");
    // Local declaration
    //    Task   *LocalTask;

    //    LocalTask = project->tasks[item->itemPos()];
    //debug("Position %d :", item->itemPos());
    //debug("Position %d :", item-> height());

    debug("taschenumero : %d",(item->itemPos()/item-> height())); 
    detailview->SetTask(project->tasks[(item->itemPos()/item-> height())]);
    debug("taschenumero : %d",(item->itemPos()/item-> height())); 
    detailview->RefreshValues();

}



void TaskView::paint( QPainter *painter )
{

	listview->setMinimumSize( 400,this->height());
	detailview->setMinimumSize( (this->width()-400),this->height() );
	//	listview->repaint();
	this->populate();
	debug("paint\n");
}

void TaskView::resizeEvent( QResizeEvent * )
{
	this->redraw();
}


